/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/

/**
 * ExampleProbeDA demonstrates how a diagnosis algorithm should
 * send probe commands to obtain more sensor data for systems
 * with probing (such as synthetic systems). See ExampleDA for
 * a more general example.
 */

#ifdef _WIN32
#include <Windows.h>
#include <Pthreads-w32/pthread.h>
#define pthreadsInit pthread_win32_process_attach_np
#define pthreadsStop pthread_win32_process_detach_np
#else
#include <unistd.h>
#include <pthread.h>
#define pthreadsInit void
#define pthreadsStop void
#endif

#include <iostream>

#include <DxcCallback.hpp>
#include <DxcDataTypes.hpp>
#include <Connection/ConnectorFactory.hpp>

using namespace Dxc;
using namespace std;

static DxcConnector *connector;
static pthread_cond_t procDataCond;
static pthread_mutex_t procDataMut;
static bool run = true;

class ExampleCallback : public DxcCallback
{
private:
    virtual void processData(const DxcData * msg)
    {
        /* msg is incoming data. Find out what type it is,
         * downcast it to that DxcData type, and process accordingly.
         */
        if (string(typeid(*msg).name()) == string(typeid(SensorData).name()))
        {
            pthread_mutex_lock(&procDataMut);
            // update observations and send diagnosis if needed

            pthread_cond_broadcast(&procDataCond);
            pthread_mutex_unlock(&procDataMut);
        }
        else if (string(typeid(*msg).name()) ==
                 string(typeid(ScenarioStatusData).name()))
        {
            // check for scenario end, exit if ended
            const ScenarioStatusData* stat =
                dynamic_cast<const ScenarioStatusData *>(msg);

            if (stat->getStatus() == ScenarioStatusData::SDS_ENDED)
            {
                run = false;
            }
        }
    };
};

int main(int argc, char **argv)
{
    ExampleCallback callback;
    pthreadsInit();
    pthread_mutex_init(&procDataMut, NULL);
    pthread_cond_init(&procDataCond, NULL);


    try
    {
        connector = ConnectorFactory::getDAConnector(&callback);
    }
    catch (...)
    {
        cerr << "DA unable to connect. Make sure ScenarioLoader " <<
             "(or StandaloneSDS) is running." << endl;
        return 1;
    }

    // signal to the system that we're prepared to receive data
    ScenarioStatusData ssd(ScenarioStatusData::DA_READY);
    connector->sendMessage(&ssd);

    while (run)
    {
        msleep(100);
    }

    // a simple strategy is to query all probe points at the end
    // of the scenario to resolve ambiguity:
    CommandSet cs;
    cs.insert(Command("probe", Value::v("z1")));
    cs.insert(Command("probe", Value::v("z2")));
    cs.insert(Command("probe", Value::v("z3")));
    // et cetera...
    CommandData cd(cs);

    // critical region, lock mutex and wait for last sensor data
    // to be processed
    pthread_mutex_lock(&procDataMut);
    connector->sendMessage(&cd);

    // wait for processData() to receive sensor data and send
    // diagnosis
    pthread_cond_wait(&procDataCond, &procDataMut);
    pthread_mutex_unlock(&procDataMut);

    // clean up after yourself!
    delete connector;

    return 0;
}
