/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.example;

import org.dxc.api.DxcCallback;
import org.dxc.api.connection.ConnectorFactory;
import org.dxc.api.connection.DxcConnector;
import org.dxc.api.datatypes.*;

/**
 * ExampleJavaRecoveryDA demonstrates how a diagnosis algorithm should query the
 * oracle for recovery actions. See ExampleDA for a more general example.
 */
public class ExampleJavaRecoveryDA {

    private static boolean run = true;
    private static RecoveryData oracleResponse = null;

    private static final DxcCallback exampleCallback = new DxcCallback() {
        public void processData(DxcData d) {
            if (d instanceof RecoveryData) {
                synchronized (this) {
                    oracleResponse = (RecoveryData) d;
                    notifyAll();
                }
            } else if (d instanceof ScenarioStatusData) {
                // check for scenario end, exit if ended

                ScenarioStatusData stat = (ScenarioStatusData) d;
                if (stat.getStatus().equals(ScenarioStatusData.ENDED))
                    run = false;
            }
        }
    };

    public static void main(String[] args) {
        try {
            DxcConnector c = ConnectorFactory.getDAConnector(exampleCallback);
            c.sendMessage(new ScenarioStatusData(ScenarioStatusData.DA_READY));

            while (run) {
                Thread.sleep(1000);
            }

            // the scenario has ended. Now assume we have two
            // candidates in our diagnosis, each with the same
            // probability--one is a load fault, the other is
            // the empty set (system is nominal):

            FaultSet fs = new FaultSet();
            StringValueMap fpm = new StringValueMap();

            fpm.put("Offset", Value.v(-2.5));
            fpm.put("FaultTime", Value.v(100.0)); // seconds since scenario
                                                  // start
            fs.add(new Fault("DC485", "ResistanceOffset", fpm));

            // the first query will return what the appropriate recovery action
            // is
            // for the load fault, and the associated cost.
            RecoveryData rd1 = new RecoveryData(fs);

            CommandSet cs = new CommandSet();
            cs.add(new Command("systemAction", Value.v("NOP"))); // no action

            // the second query will tell us the cost of taking no action if the
            // system is in the specified faulty state.
            RecoveryData rd2 = new RecoveryData(fs, cs);

            synchronized (exampleCallback) {
                c.sendMessage(rd1);
                exampleCallback.wait(10000);
                rd1 = oracleResponse;

                c.sendMessage(rd2);
                exampleCallback.wait(10000);
                rd2 = oracleResponse;
            }

            // now, since both candidates have equal probability, a
            // simple strategy is to pick the cheapest recovery action.
            if (rd1 == null || rd2 == null) {
                System.out.println("ExampleJavaDA received no RecoveryData!");
            } else if (rd1.getCost() < 0 || rd2.getCost() < 0) {
                System.out.println("ExampleJavaDA received bad RecoveryData!");
            } else {
                CommandData cmd = new CommandData();
                if (rd1.getCost() < rd2.getCost())
                    cmd.setCommands(rd1.getCommands());
                else
                    cmd.setCommands(rd2.getCommands());

                c.sendMessage(cmd);
            }

        } catch (Exception ex) {
            System.out.append(ex.toString() + " " + ex.getMessage());
        }

        System.exit(0);
    }

}
