/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef SCENARIO_DATA_HPP
#define SCENARIO_DATA_HPP

#include <typeinfo>
#include <DxcDefs.hpp>
#include <DataTypes/DxcData.hpp>

namespace Dxc
{
/**
 * @brief Data structure for scenario status messages.
 */
class DXCAPI_EXPORT ScenarioData : public DxcData
{
public:

    /**
     * @brief Constructor.
     *
     * @param timestamp - message timestamp.
     * @param systemName - system name
     * @param systemVersion - system version
     * @param frameworkVersion - framework version
     * @param scenarioName - scenario name
     * @param daName - name of DA
     * @param daVersion - DA version
     * @param scenarioLength - length of scenario
     * @param notes - miscellaneous notes
     */
    ScenarioData(Timestamp timestamp, const std::string& scenName,
                 const std::string& sysName, const std::string& sysVers,
                 const std::string& daName, const std::string& daVersion,
                 const std::string& fwVers,
                 const long len, const std::string& notes);

    /**
     * @brief Constructs with timestamp set to current time.
     *
     * @param systemName - system name
     * @param systemVersion - system version
     * @param frameworkVersion - framework version
     * @param scenarioName - scenario name
     * @param daName - name of DA
     * @param daVersion - DA version
     * @param scenarioLength - length of scenario
     * @param notes - miscellaneous notes
     */
    ScenarioData(const std::string& scenName,
                 const std::string& sysName, const std::string& sysVers,
                 const std::string& daName, const std::string& daVersion,
                 const std::string& fwVers,
                 const long len, const std::string& notes);

    /**
     * @brief Returns scenario name.
     */
    std::string getScenarioName() const
    {
        return scenName_;
    };

    /**
     * @brief Sets scenario name.
     */
    void setScenarioName(const std::string& scenName)
    {
        scenName_ = scenName;
    };

    /**
     * @brief Returns system name.
     */
    std::string getSystemName() const
    {
        return sysName_;
    };

    /**
     * @brief Sets system name.
     */
    void setSystemName(const std::string& sysName)
    {
        sysName_ = sysName;
    };

    /**
     * @brief Returns system version.
     */
    std::string getSystemVersion() const
    {
        return sysVers_;
    };

    /**
     * @brief Sets system version.
     */
    void setSystemVersion(const std::string& sysVers)
    {
        sysVers_ = sysVers;
    };

    /**
     * @brief Returns DA Name.
     */
    std::string getDAName() const
    {
        return daName_;
    };

    /**
     * @brief Sets DA Name.
     */
    void setDAName(const std::string& daName)
    {
        daName_ = daName;
    };

    /**
     * @brief Returns DA Version.
     */
    std::string getDAVersion() const
    {
        return daVers_;
    };

    /**
     * @brief Sets DA Version.
     */
    void setDAVersion(const std::string& daVers)
    {
        daVers_ = daVers;
    };

    /**
     * @brief Returns framework version.
     */
    std::string getFrameworkVersion() const
    {
        return fwVers_;
    };

    /**
     * @brief Sets framework version.
     */
    void setFrameworkVersion(const std::string& fwVers)
    {
        fwVers_ = fwVers;
    };

    /**
     * @brief Returns scenario length.
     */
    long getScenarioLength() const
    {
        return scenLen_;
    };

    /**
     * @brief Sets scenario length.
     */
    void setScenarioLength(long scenLen)
    {
        scenLen_ = scenLen;
    };

    /**
     * @brief Returns miscellaneous notes.
     */
    std::string getNotes() const
    {
        return notes_;
    };

    /**
     * @brief Sets miscellaneous notes.
     */
    void setNotes(const std::string& notes)
    {
        notes_ = notes;
    };

    virtual ScenarioData* clone() const;

    virtual std::ostream &put(std::ostream &) const;

private:
    ScenarioData();

    std::string scenName_;
    std::string sysName_;
    std::string sysVers_;
    std::string fwVers_;
    std::string daName_;
    std::string daVers_;
    std::string notes_;
    long scenLen_;
};
}

#endif
